package com.buecoin01.model;


import com.buecoin01.utility.Encryption;

import javax.persistence.*;
import java.time.LocalDateTime;
import java.util.ArrayList;


@Entity
public class Block {

    private String minerID;

    @Id
    @Column(name = "id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @Column
    private String hash;
    private String previousHash;
    //private String data;
    public String merkleRoot;
    public ArrayList<Transactions> transactionList = new ArrayList<>();
    private LocalDateTime timeStamp;
    private int nonce;

    public Block() {
    }

    public Block(String previousHash, LocalDateTime time, int nonce) {
        this.timeStamp = time;
        this.previousHash = previousHash;
        this.hash = calculateHash();
    }

    public String getHash() {
        return hash;
    }

    public String getPreviousHash() {
        return previousHash;
    }

//    public String getData() {
//        return data;
//    }

    public LocalDateTime getTimeStamp() {
        return timeStamp;
    }

    public String calculateHash() {
        return Encryption.applySha256(
                  previousHash +
                        timeStamp.toString() +
                        Integer.toString(nonce) +
                        merkleRoot);
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public void mineBlock(int difficulty) {
        String target = new String(new char[difficulty]).replace('\0', '0'); //Create a string with difficulty * "0"
        while(!hash.substring( 0, difficulty).equals(target)) {
            nonce ++;
            hash = calculateHash();
            System.out.println(hash);
        }
        System.out.println("Block Mined " + hash);
        System.out.println("You have been awarded 1 Zip!");
    }

    //add transactions to this block
    public boolean addTransaction(Transactions transactions){
        //process transaction and check if valid, unless block is genesis block then ignore
        if (transactions == null) return false;
        if (previousHash != "0"){
            if (!(transactions.processTransaction())){
                System.out.println("Transaction failed to process. Discarded.");
                return false;
            }
        }
        transactionList.add(transactions);
        System.out.println("Transaction Successfully added to the Block!");
        return true;
    }
}