package com.buecoin01.model;

import com.buecoin01.utility.Encryption;
import com.buecoin01.utility.TransactionInput;
import com.buecoin01.utility.TransactionOutput;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import javax.persistence.*;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.ArrayList;
import java.util.List;

@Component
@Entity
public class Transactions {
    @Transient
    private static int sequence = 0; //how many transactions have been generated
    @Transient
    private List <TransactionInput> inputs = new ArrayList <>();
    @Transient
    private List <TransactionOutput> outputs = new ArrayList <>();
    @Transient
    private byte[] signature;

    @Id
    @Column(name = "id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @Column
    private String transactionId; //hash of the transaction

    private PublicKey sender;
    private PublicKey reciepient;
    private String senderName;
    private String reciepientName;
    private float value;
    private String signature2;


public Transactions(){}

    public Transactions( PublicKey from, PublicKey to, float value, ArrayList <TransactionInput> inputs) {
        this.sender = from;
        this.reciepient = to;
        this.value = value;
        this.inputs = inputs;
    }

    //generating transactionId
    private String calculateHash() {
        sequence++; //increasing tho avoid identical hash codes
        return Encryption.applySha256(
                Encryption.getStringFromKey( sender ) +
                        Encryption.getStringFromKey( reciepient ) +
                        Float.toString( value ) + sequence
        );
    }


    //WRITE METHOD TO TURN SIGNATURE DEOM BYTE ARRAY TO STRING


    //Sign all the data that should not be tempered
    public void generateSignature(PrivateKey privateKey) {
        String data = Encryption.getStringFromKey( sender ) + Encryption.getStringFromKey( reciepient ) + Float.toString( value );
        signature = Encryption.applyECDSASig( privateKey, data );
    }

    //Verify the data we signed hasn't been tampered with
    public boolean verifySignature() {
        String data = Encryption.getStringFromKey( sender ) + Encryption.getStringFromKey( reciepient ) + Float.toString( value );
        return Encryption.verifyECDSASig( sender, data, signature );
    }

    //Check if transaction can be created
    public boolean processTransaction() {
        if ( !verifySignature() ) {
            System.out.println( "Transaction Signature failed to verify" );
            return false;
        }

        //gather transaction inputs to make sure they're unspent
        for (TransactionInput i : inputs) {
            i.setUTXO( BlockchainVerification.getUTXOs().get( i.getTransactionOutputId() ) );

        }
        //check if transaction is valid
        if ( getInputsValue() <  BlockchainVerification.minimumTransaction ) {
            System.out.println( "Transaction Input to small: " + getInputsValue() );
            return false;
        }
        //add outputs to Unspent list
        for (TransactionOutput o : outputs) {
            BlockchainVerification.getUTXOs().put( o.getId(), o );
        }
        //remove transaction inputs from UXTO lists as spent
        for (TransactionInput i : inputs) {
            if ( i.getUTXO() == null ) continue; // if Transaction cant be found, skip
            BlockchainVerification.getUTXOs().remove( i.getUTXO().getId() );
        }
        return true;
    }

    // returns sum of inputs(UXTOs) values
    public float getInputsValue() {
        float total = 0;
        for (TransactionInput i : inputs) {
            if ( i.getUTXO() == null ) continue; // if Transaction cant be found, skip
            total = total + i.getUTXO().getValue();
        }
        return total;
    }

    public float getOutPutsValue() {
        float total = 0;
        for (TransactionOutput o : outputs) {
            total = total + o.getValue();
        }
        return total;
    }

    // tacks in array of transactions and returns a merkle root
    public static String getMerkleRoot(ArrayList <Transactions> transactions) {
        int count = transactions.size();
        ArrayList <String> previousTreeLayer = new ArrayList <>();
        for (Transactions transaction : transactions) {
            previousTreeLayer.add( transaction.transactionId );
        }
        ArrayList <String> treeLayer = previousTreeLayer;
        while (count > 1) {
            treeLayer = new ArrayList <>();
            for (int i = 1; i < previousTreeLayer.size(); i++) {
                treeLayer.add( Encryption.applySha256( previousTreeLayer.get( i - 1 ) ) + previousTreeLayer.get( i ) );
            }
            count = treeLayer.size();
            previousTreeLayer = treeLayer;
        }
        String merkleRoot = ( treeLayer.size() == 1 ) ? treeLayer.get( 0 ) : "";
        return merkleRoot;
    }
}