package com.buecoin01.model;

import com.buecoin01.utility.TransactionInput;
import com.buecoin01.utility.TransactionOutput;

import javax.persistence.*;
import java.security.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

@Entity
public class Wallet {

    private PrivateKey privateKey;
    private PublicKey publicKey;
    public HashMap<String, TransactionOutput> UTXOs = new HashMap<>(); // only UTXOs owned by this wallet
    @Id
    @Column(name = "id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;
    @Column(name = "pubicId")
    private String publicId;
    @Column(name = "amount")
    private long amount;
    @Column(name = "name")
    private String name;


    public Wallet(String publicId, long amount, String name) {
        this.publicId = publicId;
        this.amount = amount;
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Wallet() {
        //generateKeyPair();
    }

    public PrivateKey getPrivateKey() {
        return privateKey;
    }

    public PublicKey getPublicKey() {
        return publicKey;
    }

    public long getAmount() {
        return amount;
    }

    public void setAmount(long amount) {
        this.amount = amount;
    }


    public String getPublicId() {

        return publicId;
    }

    public void setPublicId(String publicId) {
        this.publicId = publicId;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

//    public void generateKeyPair() {
//        try {
//            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("ECDSA","BC");
//            SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
//            ECGenParameterSpec ecSpec = new ECGenParameterSpec("prime192v1");
//
//            keyGen.initialize(ecSpec, random);
//            KeyPair keyPair = keyGen.generateKeyPair();
//            privateKey = keyPair.getPrivate();
//            publicKey = keyPair.getPublic();
//        } catch(Exception e) {
//            throw new RuntimeException(e);
//        }
//    }
    //returns balance and store the UTXO's owned by this wallet in this.UXTOs
    public float getBalance(){
        float total = 0;
        for (Map.Entry<String, TransactionOutput> item: BlockchainVerification.getUTXOs().entrySet()){
            TransactionOutput UTXO = item.getValue();
            if (UTXO.isMine(publicKey)){ //if output belongs to me (if coins belong to me)
                UTXOs.put(UTXO.getId(), UTXO);   // add it to our list of unspent transaction
                total = total + UTXO.getValue();

            }
        }
        return total;
    }

    //generates and returns a new transaction from this wallet


    public Transactions sendFunds(PublicKey _recipient, float value){
        if(getBalance() < value) {  //gather balance and check funds
            System.out.println("Not Enough Funds. Transaction Discarded.");
            return null;
       }

       // create arraylist of inputs
        ArrayList<TransactionInput> inputs = new ArrayList<>();
        float total = 0;
        for(Map.Entry<String, TransactionOutput> item: UTXOs.entrySet()){
            TransactionOutput UTXO = item.getValue();
            total= total + UTXO.getValue();
            inputs.add(new TransactionInput(UTXO.getId()));
            if (total > value) break;
        }
        Transactions newTransactions = new Transactions(publicKey, _recipient,value, inputs);
        newTransactions.generateSignature(privateKey);
        for (TransactionInput input : inputs){
            UTXOs.remove(input.getTransactionOutputId());
        }
        return newTransactions;
    }
}
